/**
* This sample demonstrate how to use the watchdog of the board
**/
#include "stdafx.h"

/* entry point */
int _tmain( int /*argc*/, _TCHAR* /*argv[]*/ )
{
	INT returnValue = 0;
	HANDLE boardHandle = NULL;
	ULONG readValue = 0;
	BYTE watchdogIndex = PCI3016_TIMER_2;
	BYTE enableInterrupt = 0;
	BYTE timeBase = 0;
	WORD reloadValue = 0;
	int doContinue = 1;

	/* open a board */
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d opening the board \n", returnValue);
		_getch();
		return -1;
	}

	/* display some useful informations */
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n");
	printf("* Watchdog sample. \n");
	printf("* The watchdog is used for digital outputs so we will use it.\n");
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n\n");

	/* ask for interrupts */
	ReadUlong("Do you want to enable interrupts? (0: No, 1: Yes) ",0,1, &readValue );
	enableInterrupt = (BYTE) readValue;

	/* ask for time base */
	ReadUlong("Which time base do you want to use? (0: Microseconds, 1: Milliseconds, 2: Seconds) ",0,2,&readValue);
	switch ( readValue )
	{
		case 0:	
			timeBase = PCI3016_TIME_BASE_MICROSECONDS;	
			break;
		case 1:		
			timeBase = PCI3016_TIME_BASE_MILLISECONDS;	
			break;			
		case 2:		
			timeBase = PCI3016_TIME_BASE_SECONDS;		
			break;
	}

	/* ask for reload value */
	ReadUlong("Which reload value do you want to use? (0 to 65535) ",0,0xFFFF, &readValue);
	reloadValue = (WORD) readValue;

	/* enable interrupt routine */
	if ( enableInterrupt != 0 )
	{
		returnValue = i_PCI3016_SetBoardIntRoutine( boardHandle, InterruptRoutine );
		if ( returnValue != 0 )
		{
			printf("Error %d in i_PCI3016_SetBoardIntRoutine \n", returnValue);
			i_PCI3016_CloseBoard(boardHandle);
			_getch();
			return -1;
		}
	}

	/* start the watchdog logic */
	returnValue = i_PCI3016_InitAndStartCounterTimerWatchdog( boardHandle,
		PCI3016_WATCHDOG_FUNCTION,	/* watchdog function */
		watchdogIndex,
		enableInterrupt,
		reloadValue,
		timeBase,
		0,							/* not used */
		0,							/* not used */
		0,							/* not used */
		0);							/* not used */

	/* error handling */
	if ( returnValue != 0 )
	{
		printf("error %d in i_PCI3016_InitAndStartCounterTimerWatchdog \n", returnValue);
		if ( enableInterrupt )
		{
			i_PCI3016_ResetBoardIntRoutine(boardHandle);
		}
		i_PCI3016_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	/* display some info */
	printf("i_PCI3016_StartTimer OK \n");
	printf("Press the space key to trigger the watchdog \n");
	printf("Press the enter key to write digital output 0 \n");
	printf("\n");

	/* endless loop */
	while(doContinue == 1)
	{
		BYTE watchdogStatus = 0;
		BYTE triggerStatus = 0;
		WORD watchdogValue = 0;

		/* read the status*/
		returnValue = i_PCI3016_ReadWatchdogStatus( boardHandle, &watchdogStatus, &triggerStatus, &watchdogValue);
		if ( returnValue != 0 )
		{
			printf("Error %d in i_PCI3016_StopTimer \n", returnValue);
			break;
		}

		/* print the status */
		printf("The watchdog status is %x, software trigger is %x, value : %d \r", watchdogStatus, triggerStatus, (int)watchdogValue);

		/* user hit a key ? */
		if ( _kbhit() )
		{
			switch( _getch() )
			{
				/* space */
				case 32:
					returnValue = i_PCI3016_TriggerCounterTimerWatchdog(boardHandle, watchdogIndex);
					if ( returnValue != 0 )
					{
						printf("Error %d in i_PCI3016_TriggerTimer                                            \n", returnValue);
						doContinue = 0;
					}
					printf("Timer has been triggered!                                            \n");
					break;
				/* enter */
				case 13:
					returnValue = i_PCI3016_Set1DigitalOutputOn(boardHandle, 0);
					if ( returnValue != 0 )
					{
						printf("Error %d in i_PCI3016_Set1DigitalOutputOn                                            \n", returnValue);
						doContinue = 0;
					}
					printf("Digital output 0 has been written!                                            \n");
					break;
				/* exit */
				default:
					doContinue = 0;
					break;
			}
		}
		Sleep(100);
	}


	/* stop the watchdog logic */
	returnValue = i_PCI3016_StopCounterTimerWatchdog(boardHandle, watchdogIndex);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3016_StopTimer \n", returnValue);
	}
	else
	{
		printf("i_PCI3016_StopTimer OK \n");
	}

	/* reset interrupt */
	if ( enableInterrupt != 0 )
	{
		returnValue = i_PCI3016_ResetBoardIntRoutine( boardHandle );
		if (returnValue != 0)
		{
			printf("Error %d in i_PCI3016_ResetBoardIntRoutine \n", returnValue);
		}
		else
		{
			printf("i_PCI3016_ResetBoardIntRoutine OK \n");
		}
	}

	/* close the board */
	returnValue = i_PCI3016_CloseBoard(boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3016_CloseBoard \n", returnValue);
	}
	else
	{
		printf("i_PCI3016_CloseBoard OK \n");
	}

	/* exit */
	printf("\n Press any key to exit the sample ...");
	_getch();

	return 0;
}

