using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PCI3016_NET
{
    /// <summary>
    /// Contains the call to the DLL functions
    /// </summary>
    public class DllImport
    {
        /// <summary>
        /// Definition of the function that must be given to use the interruption of the board
        /// </summary>
        /// <param name="handle"></param>
        /// <param name="fifoStatus"></param>
        /// <param name="source">Source of the interruption. 1: digital input, 2: digital output, 4: timer, 8: counter</param>
        /// <param name="mask"></param>
        /// <param name="size"></param>
        /// <param name="args"></param>
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate void InterruptFunctionEventHandler(IntPtr handle,
                                                            byte overflow,
                                                            uint source,
                                                            uint mask,
                                                            IntPtr args,
                                                            uint argsSize
                                                            );

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_GetNumberOfBoards(ref byte boardNumber );

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_OpenBoardViaIndex(byte index, ref IntPtr deviceHandle );

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_CloseBoard(IntPtr deviceHandle);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_SetBoardIntRoutine(IntPtr h_DeviceHandle, [MarshalAs(UnmanagedType.FunctionPtr)] InterruptFunctionEventHandler interruptFunction);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_ResetBoardIntRoutine(IntPtr h_DeviceHandle);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_SetDigitalOutputMemoryOn(IntPtr deviceHandle);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_SetDigitalOutputMemoryOff(IntPtr deviceHandle);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_Set1DigitalOutputOn(IntPtr deviceHandle, byte channel);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_Set1DigitalOutputOff(IntPtr deviceHandle, byte channel);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_Set4DigitalOutputsOn(IntPtr deviceHandle, byte value);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_Set4DigitalOutputsOff(IntPtr deviceHandle, byte value);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_Get1DigitalOutputStatus(IntPtr deviceHandle, byte channel, ref byte outputStatus);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_Get4DigitalOutputsStatus(IntPtr deviceHandle, ref byte outputStatus);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_Read1DigitalInput(IntPtr deviceHandle, byte channel, ref byte value);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_Read4DigitalInputs(IntPtr deviceHandle, ref byte value);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_ReadTTLInputs(IntPtr deviceHandle, byte port, ref UInt16 value);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_SetTTLPortConfiguration(IntPtr deviceHandle, byte InOrOut);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_SetTTLOutputsOff(IntPtr deviceHandle, byte port, UInt16 Mask);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_GetTTLOutputsStatus(IntPtr deviceHandle, byte port, ref UInt16 outputsStatus);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_SetTTLOutputsOn(IntPtr deviceHandle, byte port, UInt16 Mask);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_SetAnalogInputMode(IntPtr deviceHandle, byte inputMode);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_GetAnalogInputMode(IntPtr deviceHandle, ref byte inputMode);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_InitAndStartAnalogInputSequence(IntPtr deviceHandle,
            uint sequenceCount, 
            uint sequenceBeforeInterrupt,
            byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            uint convertingTime,
            uint delayTime,
            byte softTriggerMode,
            byte extTriggerMode,
            uint sequenceTriggerCount);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_StopAnalogAcquisition(IntPtr deviceHandle);

        [DllImport("PCI3016.dll")]
        public static extern int l_PCI3016_ReadSequenceData(IntPtr deviceHandle, uint size, ushort[] data);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_ReadMoreAnalogInputs(IntPtr deviceHandle,
            byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            uint convertingTime,
            byte interruptFlag,
            byte softTriggerMode,
            byte extTriggerMode,
            ushort[] value);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_Read1AnalogInput(IntPtr deviceHandle,
            byte channelIndex,
            byte gain,
            byte polarity,
            uint convertingTime,
            byte interruptFlag,
            byte softTriggerMode,
            byte extTriggerMode,
            ref ushort value);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_InitAndStartAutoRefreshAcquisition(IntPtr deviceHandle,
            byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            uint convertingTime,
            ref IntPtr value);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_TriggerAnalogAcquisition(IntPtr deviceHandle);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_GetAnalogInputChannelsNbr(IntPtr deviceHandle, ref byte analogInputCount);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_GetAnalogInputResolution(IntPtr deviceHandle, ref byte resolution);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_InitAndStartCounterTimerWatchdog(IntPtr deviceHandle,
            byte function,
            byte index,
            byte interruptFlag,
            ushort reloadValue,
            byte timerWatchdogTimeBase,
            byte counterTimerOutputMode,
            byte timerMode,
            byte counterDirection,
            byte counterLevel);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_StopCounterTimerWatchdog(IntPtr deviceHandle,
            byte index);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_TriggerCounterTimerWatchdog(IntPtr deviceHandle,
            byte index);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_ReadCounterStatus(IntPtr deviceHandle,
            byte index,
            ref byte status,
            ref byte clear,
            ref byte softwareTriggerStatus,
            ref ushort value);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_ReadCounterValue(IntPtr deviceHandle,
            byte index,
            ref ushort value);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_ClearCounter(IntPtr deviceHandle,
            byte index);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_ReadTimerStatus(IntPtr deviceHandle,
            byte index,
            ref byte status,
            ref byte softwareTriggerStatus,
            ref ushort value);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_ReadTimerValue(IntPtr deviceHandle,
            byte index,
            ref ushort value);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_ReadWatchdogStatus(IntPtr deviceHandle,
            ref byte status,
            ref byte softwareTriggerStatus,
            ref ushort value);

        [DllImport("PCI3016.dll")]
        public static extern short i_PCI3016_ReadWatchdogValue(IntPtr deviceHandle,
            ref ushort value);
    }
}
